;*******************************************************************************
; Patched (Patch D) Startup Code for D-Bug12 V2.02 Monitor Program
; for the 68HC912B32 EVB, Copyright by Motorola, Inc.
; Additions/modifications for Ram.Bo32 RAM Add-On Board by Oliver Thamm
; Assemble with ImageCraft ICC12 !
; http://www.elektronikladen.de/hc12.html
;*******************************************************************************

;
MAP_PAGE    = $0000
;
;
PORTE       = $0008+MAP_PAGE
DDRE        = $0009+MAP_PAGE
PEAR        = $000a+MAP_PAGE
MODE        = $000b+MAP_PAGE
PEAR_MODE   = PEAR                      ; PEAR:MODE for word access
INITRG      = $0011+MAP_PAGE
INITEE      = $0012+MAP_PAGE
MISC        = $0013+MAP_PAGE
COPCTL      = $0016+MAP_PAGE
CSCTL0      = $003c+MAP_PAGE
CSCTL1      = $003d+MAP_PAGE
CSSTR0      = $003e+MAP_PAGE
PORTAD      = $006f+MAP_PAGE
PORTS       = $00d6+MAP_PAGE
DDRS        = $00d7+MAP_PAGE
EEMCR       = $00f0+MAP_PAGE
BPROT       = $00f1+MAP_PAGE
FEELCK      = $00f4+MAP_PAGE
;
MonRAMStart = $0A00
MonRAMSize  = $0200

RAM_START   = $0800
RAMSize     = $0400
STACKTOP    = RAM_START+RAMSize         ; stack at top of internal RAM

EE_START    = $0d00                     ; EEPROM located here out of reset

;--------
_UserFnTable = $F680                    ; xref _UserFnTable
;--------

WP			= $04						; PORTS[2] -> $8000+ Write Disable
A15CTRL     = $08                       ; PORTS[3] -> RAM[A15]

;*******************************************************************************

            .area memory(abs)
            .org 0xf6c0

CustData::

UserCCR:        .db $90                 ; User CPU Condition Code Register
UserB:          .db $00                 ; User CPU B-accumulator
UserA:          .db $00                 ; User CPU A-accumulator
UserX:          .dw $0000               ; User CPU X-index register
UserY:          .dw $0000               ; User CPU Y-index register
UserPC:         .dw $0000               ; User CPU Program Counter
UserSP:         .dw $0A00               ; User CPU Stack Pointer
SysClk:         .dw $007a
                .dw $1200               ; System Clock frequency (8000000 Hz)
IOBase:         .dw $0000               ; Base address of the I/O registers
SCIBaudRegVal:  .dw 52                  ; Initial SCI BAUD register value
EEBase:         .dw $0D00               ; Base address of the on-chip EEPROM
EESize:         .dw 768                 ; size of the on-chip EEPROM
EEDelay:        .dw _EEDelay            ; address of EEPROM program/erase delay routine
AuxCmdCount:    .dw 0                   ; number of commands in auxiliary command table
AuxCmdTableP:   .dw $0000               ; pointer to the auxiliary command table


;*******************************************************************************
; INITIALIZATION
;
; The code in this section is initialization for the monitor on the EVB12B32
;*******************************************************************************

            .area memory(abs)
            .org 0xf700

__MonStartup:
            lds #STACKTOP               ; initialize monitor stack pointer
;
;   Disable the COP watchdog by CR2:CR1:CR0 = 0:0:0
;   COPCTL = $07 when reset in normal modes
;   FCME and CRx bits are write once in normal modes
;   COPCTL [ CME :FCME : FCM : FCOP ! DISR : CR2 : CR1 : CR0 ] $--16
;
            clr COPCTL                  ; disable watchdog

;   Enable pipe signals, E, low strobe and read/write in port E
;   PIPOE, NECLK, LSTRE and RDWE are write once in normal modes
;   PEAR [NDBE : 0 : PIPOE : NECLK ! LSTRE : RDWE : 0 : 0 ] $--0A
;
;   Change to Normal Expanded Wide Mode, enable internal visibility
;   MODE [SMODN : MODB : MODA : ESTR ! IVIS : 0 : EMD : EME ] $--0B
;
            movw #$2cf8,PEAR_MODE

;   Copy Flash contents to RAM
;
            bset  DDRS,WP               ; WP becomes an output
            bclr  PORTS,WP              ; WP = L (enable RAM Write)
            ;
            ldx   #$8000                ; read from Flash (internal)
_flash2ram: movw  0,x, 2,x+             ; write to RAM (external)
            tbne  x, _flash2ram         ; loop if (X<=$FFFF)
            ;
            bset  PORTS,WP              ; WP = H (protect RAM)

;   Remove Flash from memory map (continue exec in RAM); No clock stretching
;   MISC [ 0 : NDRF : RFSTR1 : RFSTR0 ! EXSTR1 : EXSTR0 : MAPROM : ROMON ] $--13
;
            ldaa #$02
            staa *MISC                  ; Flash OFF

;   Clear all monitor RAM to start from a known state
;
            ldx #MonRAMStart
ClrRAM:     clr 1,x+                    ; clear one and inc pointer
            cpx #MonRAMStart+MonRAMSize
            bne ClrRAM                  ; loop till RAM clear

;   Enable EEPROM so monitor can program/erase bytes
;   EEMCR [ 1 : 1 : 1 : 1 ! 1 : 1 :PROTLK: EERC ] $--F0
;   BPROT [ 1 :BPROT6:BPROT5:BPROT4!BPROT3:BPROT2:BPROT1:BPROT0] $--F1
;
            ldaa #$fc                   ; prevent later protection lock
            staa *EEMCR                 ; PROTLK is write-once
            clr BPROT                   ; allow EE program and erase
;
;   Disable writing to the on-chip Flash EEPROM
;   FEELCK [ 0 : 0 : 0 : 0 : 0 : 0 : 0 : LOCK] $--F4
;
            ldaa #$01                   ; write a 1 to the Flash LOCK bit to
                                        ; disable accidential reprogramming of
            staa *FEELCK                ; the flash memory (where we're located)
;
            ldd EEBase                  ; get the user supplied base address of the
                                        ; on-chip EEPROM

            oraa #1                     ; make sure that the EEON bit remains set.
            staa *INITEE                ; re-map the on-chip EEPROM.
;
            ldd IOBase                  ; get the user supplied base address of the
                                        ; on-chip I/O registers
            staa *INITRG                ; re-map the on-chip registers.
            ldx #_UserFnTable           ; point to the table of user accessible
                                        ; routines.
            jmp [0,x]                   ; the first entry is a pointer to main.
                                        ; GO.........
;
;
;
;   This small subroutine is used to produce a delay of approximately 10 mS.
;   This delay is based on the following conditions:
;
;   1.) An 8.00 MHz E-clock
;   2.) Subroutine located in internal memory
;
;   This routine is called by D-Bug12's WriteEEByte() function (through a
;   pointer stored in the Customization Data Table).
;
_EEDelay:
            ldx #20000                  ; load delay count into x
DlyLoop:    dex                         ; decrement count
            bne DlyLoop                 ; loop till done.
            rts                         ; return.
;
;
;-- END ------------------------------------------------------------------------
